@@ -24,6 +24,7 @@ import java.util.Date;
 import java.util.Iterator;
 import java.util.Locale;
 import java.util.NoSuchElementException;
+import java.util.concurrent.TimeUnit;
 
 /**
  * <p>A suite of utilities surrounding the use of the
@@ -41,7 +42,7 @@ import java.util.NoSuchElementException;
  * </p>
  *
  * @since 2.0
- * @version $Id: DateUtils.java 1436770 2013-01-22 07:09:45Z ggregory $
+ * @version $Id: DateUtils.java 1583482 2014-03-31 22:54:57Z niallp $
  */
 public class DateUtils {
 
@@ -347,7 +348,7 @@ public class DateUtils {
      * @return the parsed date
      * @throws IllegalArgumentException if the date string or pattern array is null
      * @throws ParseException if none of the date patterns were suitable
-     * @see java.util.Calender#isLenient()
+     * @see java.util.Calendar#isLenient()
      */
     private static Date parseDateWithLeniency(
             final String str, final Locale locale, final String[] parsePatterns, final boolean lenient) throws ParseException {
@@ -683,13 +684,13 @@ public class DateUtils {
      * saving time, rounding to Calendar.HOUR_OF_DAY will behave as follows.
      * Suppose daylight saving time begins at 02:00 on March 30. Rounding a 
      * date that crosses this time would produce the following values:
+     * </p>
      * <ul>
      * <li>March 30, 2003 01:10 rounds to March 30, 2003 01:00</li>
      * <li>March 30, 2003 01:40 rounds to March 30, 2003 03:00</li>
      * <li>March 30, 2003 02:10 rounds to March 30, 2003 03:00</li>
      * <li>March 30, 2003 02:40 rounds to March 30, 2003 04:00</li>
      * </ul>
-     * </p>
      * 
      * @param date  the date to work with, not null
      * @param field  the field from {@code Calendar} or {@code SEMI_MONTH}
@@ -719,13 +720,13 @@ public class DateUtils {
      * saving time, rounding to Calendar.HOUR_OF_DAY will behave as follows.
      * Suppose daylight saving time begins at 02:00 on March 30. Rounding a 
      * date that crosses this time would produce the following values:
+     * </p>
      * <ul>
      * <li>March 30, 2003 01:10 rounds to March 30, 2003 01:00</li>
      * <li>March 30, 2003 01:40 rounds to March 30, 2003 03:00</li>
      * <li>March 30, 2003 02:10 rounds to March 30, 2003 03:00</li>
      * <li>March 30, 2003 02:40 rounds to March 30, 2003 04:00</li>
      * </ul>
-     * </p>
      * 
      * @param date  the date to work with, not null
      * @param field  the field from {@code Calendar} or <code>SEMI_MONTH</code>
@@ -755,13 +756,13 @@ public class DateUtils {
      * saving time, rounding to Calendar.HOUR_OF_DAY will behave as follows.
      * Suppose daylight saving time begins at 02:00 on March 30. Rounding a 
      * date that crosses this time would produce the following values:
+     * </p>
      * <ul>
      * <li>March 30, 2003 01:10 rounds to March 30, 2003 01:00</li>
      * <li>March 30, 2003 01:40 rounds to March 30, 2003 03:00</li>
      * <li>March 30, 2003 02:10 rounds to March 30, 2003 03:00</li>
      * <li>March 30, 2003 02:40 rounds to March 30, 2003 04:00</li>
      * </ul>
-     * </p>
      * 
      * @param date  the date to work with, either {@code Date} or {@code Calendar}, not null
      * @param field  the field from {@code Calendar} or <code>SEMI_MONTH</code>
@@ -1073,6 +1074,8 @@ public class DateUtils {
                         offsetSet = true;
                     }
                     break;
+                default:
+                    break;
             }
             if (!offsetSet) {
                 final int min = val.getActualMinimum(aField[0]);
@@ -1195,6 +1198,8 @@ public class DateUtils {
                         startCutoff = focus.get(Calendar.DAY_OF_WEEK) - 3;
                         endCutoff = focus.get(Calendar.DAY_OF_WEEK) + 3;
                         break;
+                    default:
+                        break;
                 }
                 break;
             default:
@@ -1266,7 +1271,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a SECOND field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.SECOND as fragment will return 538</li>
      *  <li>January 6, 2008 7:15:10.538 with Calendar.SECOND as fragment will return 538</li>
@@ -1274,7 +1278,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in milliseconds)</li>
      * </ul>
-     * </p>
      * 
      * @param date the date to work with, not null
      * @param fragment the {@code Calendar} field part of date to calculate 
@@ -1284,7 +1287,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInMilliseconds(final Date date, final int fragment) {
-        return getFragment(date, fragment, Calendar.MILLISECOND);    
+        return getFragment(date, fragment, TimeUnit.MILLISECONDS);    
     }
     
     /**
@@ -1303,7 +1306,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a SECOND field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.MINUTE as fragment will return 10
      *   (equivalent to deprecated date.getSeconds())</li>
@@ -1314,7 +1316,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in seconds)</li>
      * </ul>
-     * </p>
      * 
      * @param date the date to work with, not null
      * @param fragment the {@code Calendar} field part of date to calculate 
@@ -1324,7 +1325,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInSeconds(final Date date, final int fragment) {
-        return getFragment(date, fragment, Calendar.SECOND);
+        return getFragment(date, fragment, TimeUnit.SECONDS);
     }
     
     /**
@@ -1343,7 +1344,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a MINUTE field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.HOUR_OF_DAY as fragment will return 15
      *   (equivalent to deprecated date.getMinutes())</li>
@@ -1354,7 +1354,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in minutes)</li>
      * </ul>
-     * </p>
      * 
      * @param date the date to work with, not null
      * @param fragment the {@code Calendar} field part of date to calculate 
@@ -1364,7 +1363,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInMinutes(final Date date, final int fragment) {
-        return getFragment(date, fragment, Calendar.MINUTE);
+        return getFragment(date, fragment, TimeUnit.MINUTES);
     }
     
     /**
@@ -1383,7 +1382,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a HOUR field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.DAY_OF_YEAR as fragment will return 7
      *   (equivalent to deprecated date.getHours())</li>
@@ -1394,7 +1392,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in hours)</li>
      * </ul>
-     * </p>
      * 
      * @param date the date to work with, not null
      * @param fragment the {@code Calendar} field part of date to calculate 
@@ -1404,7 +1401,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInHours(final Date date, final int fragment) {
-        return getFragment(date, fragment, Calendar.HOUR_OF_DAY);
+        return getFragment(date, fragment, TimeUnit.HOURS);
     }
     
     /**
@@ -1423,7 +1420,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a DAY field will return 0.</p> 
      *  
-     * <p>
      * <ul>
      *  <li>January 28, 2008 with Calendar.MONTH as fragment will return 28
      *   (equivalent to deprecated date.getDay())</li>
@@ -1434,7 +1430,6 @@ public class DateUtils {
      *  <li>January 28, 2008 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in days)</li>
      * </ul>
-     * </p>
      * 
      * @param date the date to work with, not null
      * @param fragment the {@code Calendar} field part of date to calculate 
@@ -1444,7 +1439,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInDays(final Date date, final int fragment) {
-        return getFragment(date, fragment, Calendar.DAY_OF_YEAR);
+        return getFragment(date, fragment, TimeUnit.DAYS);
     }
 
     /**
@@ -1463,7 +1458,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a MILLISECOND field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.SECOND as fragment will return 538
      *   (equivalent to calendar.get(Calendar.MILLISECOND))</li>
@@ -1474,7 +1468,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in milliseconds)</li>
      * </ul>
-     * </p>
      * 
      * @param calendar the calendar to work with, not null
      * @param fragment the {@code Calendar} field part of calendar to calculate 
@@ -1484,7 +1477,7 @@ public class DateUtils {
      * @since 2.4
      */
   public static long getFragmentInMilliseconds(final Calendar calendar, final int fragment) {
-    return getFragment(calendar, fragment, Calendar.MILLISECOND);
+    return getFragment(calendar, fragment, TimeUnit.MILLISECONDS);
   }
     /**
      * <p>Returns the number of seconds within the 
@@ -1502,7 +1495,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a SECOND field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.MINUTE as fragment will return 10
      *   (equivalent to calendar.get(Calendar.SECOND))</li>
@@ -1513,7 +1505,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in seconds)</li>
      * </ul>
-     * </p>
      * 
      * @param calendar the calendar to work with, not null
      * @param fragment the {@code Calendar} field part of calendar to calculate 
@@ -1523,7 +1514,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInSeconds(final Calendar calendar, final int fragment) {
-        return getFragment(calendar, fragment, Calendar.SECOND);
+        return getFragment(calendar, fragment, TimeUnit.SECONDS);
     }
     
     /**
@@ -1542,7 +1533,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a MINUTE field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.HOUR_OF_DAY as fragment will return 15
      *   (equivalent to calendar.get(Calendar.MINUTES))</li>
@@ -1553,7 +1543,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in minutes)</li>
      * </ul>
-     * </p>
      * 
      * @param calendar the calendar to work with, not null
      * @param fragment the {@code Calendar} field part of calendar to calculate 
@@ -1563,7 +1552,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInMinutes(final Calendar calendar, final int fragment) {
-        return getFragment(calendar, fragment, Calendar.MINUTE);
+        return getFragment(calendar, fragment, TimeUnit.MINUTES);
     }
     
     /**
@@ -1582,7 +1571,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a HOUR field will return 0.</p> 
      *  
-     * <p>
      * <ul>
      *  <li>January 1, 2008 7:15:10.538 with Calendar.DAY_OF_YEAR as fragment will return 7
      *   (equivalent to calendar.get(Calendar.HOUR_OF_DAY))</li>
@@ -1593,7 +1581,6 @@ public class DateUtils {
      *  <li>January 16, 2008 7:15:10.538 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in hours)</li>
      * </ul>
-     * </p>
      *  
      * @param calendar the calendar to work with, not null
      * @param fragment the {@code Calendar} field part of calendar to calculate 
@@ -1603,7 +1590,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInHours(final Calendar calendar, final int fragment) {
-        return getFragment(calendar, fragment, Calendar.HOUR_OF_DAY);
+        return getFragment(calendar, fragment, TimeUnit.HOURS);
     }
     
     /**
@@ -1622,7 +1609,6 @@ public class DateUtils {
      * Calendar.MINUTE, Calendar.SECOND and Calendar.MILLISECOND
      * A fragment less than or equal to a DAY field will return 0.</p> 
      * 
-     * <p>
      * <ul>
      *  <li>January 28, 2008 with Calendar.MONTH as fragment will return 28
      *   (equivalent to calendar.get(Calendar.DAY_OF_MONTH))</li>
@@ -1635,7 +1621,6 @@ public class DateUtils {
      *  <li>January 28, 2008 with Calendar.MILLISECOND as fragment will return 0
      *   (a millisecond cannot be split in days)</li>
      * </ul>
-     * </p>
      * 
      * @param calendar the calendar to work with, not null
      * @param fragment the {@code Calendar} field part of calendar to calculate 
@@ -1645,7 +1630,7 @@ public class DateUtils {
      * @since 2.4
      */
     public static long getFragmentInDays(final Calendar calendar, final int fragment) {
-        return getFragment(calendar, fragment, Calendar.DAY_OF_YEAR);
+        return getFragment(calendar, fragment, TimeUnit.DAYS);
     }
     
     /**
@@ -1653,13 +1638,13 @@ public class DateUtils {
      * 
      * @param date the date to work with, not null
      * @param fragment the Calendar field part of date to calculate 
-     * @param unit the {@code Calendar} field defining the unit
+     * @param unit the time unit
      * @return number of units within the fragment of the date
      * @throws IllegalArgumentException if the date is <code>null</code> or 
      * fragment is not supported
      * @since 2.4
      */
-    private static long getFragment(final Date date, final int fragment, final int unit) {
+    private static long getFragment(final Date date, final int fragment, final TimeUnit unit) {
         if(date == null) {
             throw  new IllegalArgumentException("The date must not be null");
         }
@@ -1673,26 +1658,30 @@ public class DateUtils {
      * 
      * @param calendar the calendar to work with, not null
      * @param fragment the Calendar field part of calendar to calculate 
-     * @param unit the {@code Calendar} field defining the unit
+     * @param unit the time unit
      * @return number of units within the fragment of the calendar
      * @throws IllegalArgumentException if the date is <code>null</code> or 
      * fragment is not supported
      * @since 2.4
      */
-    private static long getFragment(final Calendar calendar, final int fragment, final int unit) {
+    private static long getFragment(final Calendar calendar, final int fragment, final TimeUnit unit) {
         if(calendar == null) {
             throw  new IllegalArgumentException("The date must not be null"); 
         }
-        final long millisPerUnit = getMillisPerUnit(unit);
+
         long result = 0;
         
+        int offset = (unit == TimeUnit.DAYS) ? 0 : 1;
+        
         // Fragments bigger than a day require a breakdown to days
         switch (fragment) {
             case Calendar.YEAR:
-                result += (calendar.get(Calendar.DAY_OF_YEAR) * MILLIS_PER_DAY) / millisPerUnit;
+                result += unit.convert(calendar.get(Calendar.DAY_OF_YEAR) - offset, TimeUnit.DAYS);
                 break;
             case Calendar.MONTH:
-                result += (calendar.get(Calendar.DAY_OF_MONTH) * MILLIS_PER_DAY) / millisPerUnit;
+                result += unit.convert(calendar.get(Calendar.DAY_OF_MONTH) - offset, TimeUnit.DAYS);
+                break;
+            default:
                 break;
         }
 
@@ -1704,16 +1693,16 @@ public class DateUtils {
             // The rest of the valid cases
             case Calendar.DAY_OF_YEAR:
             case Calendar.DATE:
-                result += (calendar.get(Calendar.HOUR_OF_DAY) * MILLIS_PER_HOUR) / millisPerUnit;
+                result += unit.convert(calendar.get(Calendar.HOUR_OF_DAY), TimeUnit.HOURS);
                 //$FALL-THROUGH$
             case Calendar.HOUR_OF_DAY:
-                result += (calendar.get(Calendar.MINUTE) * MILLIS_PER_MINUTE) / millisPerUnit;
+                result += unit.convert(calendar.get(Calendar.MINUTE), TimeUnit.MINUTES);
                 //$FALL-THROUGH$
             case Calendar.MINUTE:
-                result += (calendar.get(Calendar.SECOND) * MILLIS_PER_SECOND) / millisPerUnit;
+                result += unit.convert(calendar.get(Calendar.SECOND), TimeUnit.SECONDS);
                 //$FALL-THROUGH$
             case Calendar.SECOND:
-                result += (calendar.get(Calendar.MILLISECOND) * 1) / millisPerUnit;
+                result += unit.convert(calendar.get(Calendar.MILLISECOND), TimeUnit.MILLISECONDS);
                 break;
             case Calendar.MILLISECOND: break;//never useful
                 default: throw new IllegalArgumentException("The fragment " + fragment + " is not supported");
@@ -1795,38 +1784,6 @@ public class DateUtils {
         return truncatedDate1.compareTo(truncatedDate2);
     }
 
-    /**
-     * Returns the number of milliseconds of a {@code Calendar} field, if this is a constant value.
-     * This handles millisecond, second, minute, hour and day (even though days can very in length).
-     * 
-     * @param unit  a {@code Calendar} field constant which is a valid unit for a fragment
-     * @return the number of milliseconds in the field
-     * @throws IllegalArgumentException if date can't be represented in milliseconds
-     * @since 2.4 
-     */
-    private static long getMillisPerUnit(final int unit) {
-        long result = Long.MAX_VALUE;
-        switch (unit) {
-            case Calendar.DAY_OF_YEAR:
-            case Calendar.DATE:
-                result = MILLIS_PER_DAY;
-                break;
-            case Calendar.HOUR_OF_DAY:
-                result = MILLIS_PER_HOUR;
-                break;
-            case Calendar.MINUTE:
-                result = MILLIS_PER_MINUTE;
-                break;
-            case Calendar.SECOND:
-                result = MILLIS_PER_SECOND;
-                break;
-            case Calendar.MILLISECOND:
-                result = 1;
-                break;
-            default: throw new IllegalArgumentException("The unit " + unit + " cannot be represented is milleseconds");
-        }
-        return result;
-    }
 
     //-----------------------------------------------------------------------
     /**
