@@ -63,7 +63,7 @@ import org.apache.commons.lang3.Validate;
  * 'YYY' will be formatted as '2003', while it was '03' in former Java
  * versions. FastDatePrinter implements the behavior of Java 7.</p>
  *
- * @version $Id: FastDatePrinter.java 1552661 2013-12-20 13:36:30Z britter $
+ * @version $Id: FastDatePrinter.java 1567799 2014-02-12 23:25:58Z sebb $
  * @since 3.2
  */
 public class FastDatePrinter implements DatePrinter, Serializable {
@@ -1085,7 +1085,7 @@ public class FastDatePrinter implements DatePrinter, Serializable {
 
     //-----------------------------------------------------------------------
 
-    private static ConcurrentMap<TimeZoneDisplayKey, String> cTimeZoneDisplayCache =
+    private static final ConcurrentMap<TimeZoneDisplayKey, String> cTimeZoneDisplayCache =
         new ConcurrentHashMap<TimeZoneDisplayKey, String>(7);
     /**
      * <p>Gets the time zone display name, using a cache for performance.</p>
@@ -1233,12 +1233,13 @@ public class FastDatePrinter implements DatePrinter, Serializable {
          * @param locale the timezone locale
          */
         TimeZoneDisplayKey(final TimeZone timeZone,
-                           final boolean daylight, int style, final Locale locale) {
+                           final boolean daylight, final int style, final Locale locale) {
             mTimeZone = timeZone;
             if (daylight) {
-                style |= 0x80000000;
+                mStyle = style | 0x80000000;
+            } else {
+                mStyle = style;
             }
-            mStyle = style;
             mLocale = locale;
         }
 
