@@ -41,7 +41,7 @@ import org.junit.Test;
 /**
  * Unit tests for {@link LocaleUtils}.
  *
- * @version $Id: LocaleUtilsTest.java 1555044 2014-01-03 10:27:30Z britter $
+ * @version $Id: LocaleUtilsTest.java 1582699 2014-03-28 11:15:32Z sebb $
  */
 public class LocaleUtilsTest  {
 
@@ -51,9 +51,7 @@ public class LocaleUtilsTest  {
     private static final Locale LOCALE_FR = new Locale("fr", "");
     private static final Locale LOCALE_FR_CA = new Locale("fr", "CA");
     private static final Locale LOCALE_QQ = new Locale("qq", "");
-    private static final Locale LOCALE_QQ_ZZ = new Locale("qq", "ZZ");
-
-
+    private static final Locale LOCALE_QQ_ZZ = new Locale("qq", "ZZ"); 
 
     @Before
     public void setUp() throws Exception {
@@ -161,11 +159,6 @@ public class LocaleUtilsTest  {
             LocaleUtils.toLocale("u");
             fail("Must be 2 chars if less than 5");
         } catch (final IllegalArgumentException iae) {}
-       
-        try {
-            LocaleUtils.toLocale("uuu");
-            fail("Must be 2 chars if less than 5");
-        } catch (final IllegalArgumentException iae) {}
 
         try {
             LocaleUtils.toLocale("uu_U");
@@ -378,6 +371,21 @@ public class LocaleUtilsTest  {
         assertEquals(set.contains(LOCALE_QQ), LocaleUtils.isAvailableLocale(LOCALE_QQ));
         assertEquals(set.contains(LOCALE_QQ_ZZ), LocaleUtils.isAvailableLocale(LOCALE_QQ_ZZ));
     }
+    
+    /**
+     * Test for 3-chars locale, further details at LANG-915
+     * 
+     */
+    @Test
+    public void testThreeCharsLocale() {
+        for (String str : Arrays.asList("udm", "tet")) {
+            Locale locale = LocaleUtils.toLocale(str);
+            assertNotNull(locale);
+            assertEquals(str, locale.getLanguage());
+            assertTrue(StringUtils.isBlank(locale.getCountry()));
+            assertEquals(new Locale(str), locale);
+        }
+    }
 
     //-----------------------------------------------------------------------
     /**
